@@ -4,13 +4,14 @@
 #include "llvm-c/TargetMachine.h"
 #include "llvm/Target/TargetMachine.h"
 #include "llvm/IR/LegacyPassManager.h"
-#include "llvm/Target/TargetLibraryInfo.h"
+#include "llvm/Analysis/TargetLibraryInfo.h"
 #include "llvm/ADT/Triple.h"
 #include "llvm/Support/TargetRegistry.h"
 #include "llvm/IR/Type.h"
 
 #include <cstdio>
 #include <cstring>
+#include <sstream>
 
 
 namespace llvm {
@@ -36,9 +37,33 @@ inline LLVMTargetMachineRef wrap(TargetMachine *TM) {
 
 extern "C" {
 
+API_EXPORT(void)
+LLVMPY_GetProcessTriple(const char **Out) {
+    *Out = LLVMPY_CreateString(llvm::sys::getProcessTriple().c_str());
+}
+
+/**
+ * Output the feature string to the output argument.
+ * Features are prefixed with '+' or '-' for enabled or disabled, respectively.
+ * Features are separated by ','.
+ */
+API_EXPORT(void)
+LLVMPY_GetHostCPUFeatures(const char **Out){
+    llvm::StringMap<bool> features;
+    std::ostringstream buf;
+    if ( llvm::sys::getHostCPUFeatures(features) ) {
+        for (auto &F : features) {
+            if (buf.tellp()){
+                buf << ',';
+            }
+            buf << ((F.second? "+": "-") + F.first()).str();
+        }
+    }
+    *Out = LLVMPY_CreateString(buf.str().c_str());
+}
+
 API_EXPORT(void)
 LLVMPY_GetDefaultTargetTriple(const char **Out) {
-    // Should we use getProcessTriple() instead?
     *Out = LLVMPY_CreateString(llvm::sys::getDefaultTargetTriple().c_str());
 }
 
@@ -195,7 +220,7 @@ LLVMPY_CreateTargetMachine(LLVMTargetRef T,
         rm = Reloc::Default;
 
     TargetOptions opt;
-    opt.JITEmitDebugInfo = EmitJITDebug;
+//     opt.JITEmitDebugInfo = EmitJITDebug;
     opt.PrintMachineCode = PrintMC;
 
     return wrap(unwrap(T)->createTargetMachine(Triple, CPU, Features, opt,
@@ -276,53 +301,16 @@ LLVMPY_DisposeMemoryBuffer(LLVMMemoryBufferRef MB)
     return LLVMDisposeMemoryBuffer(MB);
 }
 
+/*
 
-API_EXPORT(LLVMTargetLibraryInfoRef)
-LLVMPY_CreateTargetLibraryInfo(const char *Triple)
-{
-    return llvm::wrap(new llvm::TargetLibraryInfo(llvm::Triple(Triple)));
+If needed:
+
+explicit TargetLibraryInfoWrapperPass(const Triple &T);
+
+void LLVMAddTargetLibraryInfo(LLVMTargetLibraryInfoRef TLI,
+                              LLVMPassManagerRef PM) {
+  unwrap(PM)->add(new TargetLibraryInfoWrapperPass(*unwrap(TLI)));
 }
-
-API_EXPORT(void)
-LLVMPY_DisposeTargetLibraryInfo(LLVMTargetLibraryInfoRef TLI)
-{
-    delete llvm::unwrap(TLI);
-}
-
-API_EXPORT(void)
-LLVMPY_AddTargetLibraryInfo(
-    LLVMTargetLibraryInfoRef TLI,
-    LLVMPassManagerRef PM
-    )
-{
-    LLVMAddTargetLibraryInfo(TLI, PM);
-}
-
-API_EXPORT(void)
-LLVMPY_DisableAllBuiltins(LLVMTargetLibraryInfoRef TLI)
-{
-    llvm::unwrap(TLI)->disableAllFunctions();
-}
-
-API_EXPORT(int)
-LLVMPY_GetLibFunc(LLVMTargetLibraryInfoRef TLI, const char *Name, int *OutF)
-{
-    llvm::LibFunc::Func F;
-    if (llvm::unwrap(TLI)->getLibFunc(Name, F)) {
-        /* Ok */
-        *OutF = F;
-        return 1;
-    } else {
-        /* Failed */
-        return 0;
-    }
-}
-
-API_EXPORT(void)
-LLVMPY_SetUnavailableLibFunc(LLVMTargetLibraryInfoRef TLI, int F)
-{
-    llvm::unwrap(TLI)->setUnavailable((llvm::LibFunc::Func)F);
-}
-
+*/
 
 } // end extern "C"
