@@ -67,12 +67,18 @@ class ExecutionEngine(ffi.ObjectRef):
         return ptr
 
     def get_function_address(self, name):
+        """
+        Return the address of the function named *name* as an integer.
+        """
         ptr = ffi.lib.LLVMPY_GetFunctionAddress(self, name.encode("ascii"))
         if ptr is None:
             raise ValueError("Cannot find given function %s" % name)
         return ptr
 
     def get_global_value_address(self, name):
+        """
+        Return the address of the global value named *name* as an integer.
+        """
         ptr = ffi.lib.LLVMPY_GetGlobalValueAddress(self, name.encode("ascii"))
         if ptr is None:
             raise ValueError("Cannot find given global value %s" % name)
@@ -93,6 +99,10 @@ class ExecutionEngine(ffi.ObjectRef):
         self._modules.add(module)
 
     def finalize_object(self):
+        """
+        Make sure all modules owned by the execution engine are fully processed
+        and "usable" for execution.
+        """
         ffi.lib.LLVMPY_FinalizeObject(self)
 
     def remove_module(self, module):
