@@ -6,10 +6,63 @@ from ctypes import (POINTER, c_char_p, c_longlong, c_int, c_size_t,
                     c_void_p, string_at, byref)
 
 from . import ffi
-from .module import parse_assembly
 from .common import _decode_string, _encode_string
 
 
+def get_process_triple():
+    """
+    Return a target triple suitable for generating code for the current process.
+    An example when the default triple from ``get_default_triple()`` is not be
+    suitable is when LLVM is compiled for 32-bit but the process is executing
+    in 64-bit mode.
+    """
+    with ffi.OutputString() as out:
+        ffi.lib.LLVMPY_GetProcessTriple(out)
+        return str(out)
+
+
+class FeatureMap(dict):
+    """
+    Maps feature name to a boolean indicating the availability of the feature.
+    Extends ``dict`` to add `.flatten()` method.
+    """
+    def flatten(self, sort=True):
+        """
+        Args
+        ----
+        sort: bool
+            Optional.  If True, the features are sorted by name; otherwise,
+            the ordering is unstable between python session due to hash
+            randomization.  Defaults to True.
+
+        Returns a string suitable for use as the ``features`` argument to
+        ``Target.create_target_machine()``.
+
+        """
+        iterator = sorted(self.items()) if sort else iter(self.items())
+        flag_map = {True: '+', False: '-'}
+        return ','.join('{0}{1}'.format(flag_map[v], k)
+                        for k, v in iterator)
+
+
+def get_host_cpu_features():
+    """
+    Returns a dictionary-like object indicating the CPU features for current
+    architecture and whether they are enabled for this CPU.  The key-value pairs
+    are the feature name as string and a boolean indicating whether the feature
+    is available.  The returned value is an instance of ``FeatureMap`` class,
+    which adds a new method ``.flatten()`` for returning a string suitable for
+    use as the "features" argument to ``Target.create_target_machine()``.
+    """
+    with ffi.OutputString() as out:
+        ffi.lib.LLVMPY_GetHostCPUFeatures(out)
+        outdict = FeatureMap()
+        flag_map = {'+': True, '-': False}
+        for feat in str(out).split(','):
+            outdict[feat[1:]] = flag_map[feat[0]]
+        return outdict
+
+
 def get_default_triple():
     """
     Return the default target triple LLVM is configured to produce code for.
@@ -45,8 +98,11 @@ def get_object_format(triple=None):
     return _object_formats[res]
 
 
-def create_target_data(strrep):
-    return TargetData(ffi.lib.LLVMPY_CreateTargetData(_encode_string(strrep)))
+def create_target_data(layout):
+    """
+    Create a TargetData instance for the given *layout* string.
+    """
+    return TargetData(ffi.lib.LLVMPY_CreateTargetData(_encode_string(layout)))
 
 
 class TargetData(ffi.ObjectRef):
@@ -111,11 +167,17 @@ class Target(ffi.ObjectRef):
 
     @classmethod
     def from_default_triple(cls):
+        """
+        Create a Target instance for the default triple.
+        """
         triple = get_default_triple()
         return cls.from_triple(triple)
 
     @classmethod
     def from_triple(cls, triple):
+        """
+        Create a Target instance for the given triple (a string).
+        """
         with ffi.OutputString() as outerr:
             target = ffi.lib.LLVMPY_GetTargetFromTriple(triple.encode('utf8'),
                                                         outerr)
@@ -145,6 +207,9 @@ class Target(ffi.ObjectRef):
     def create_target_machine(self, cpu='', features='',
                               opt=2, reloc='default', codemodel='jitdefault',
                               jitdebug=False, printmc=False):
+        """
+        Create a new TargetMachine for this target and the given options.
+        """
         assert 0 <= opt <= 3
         assert reloc in RELOC
         assert codemodel in CODEMODEL
@@ -231,57 +296,13 @@ class TargetMachine(ffi.ObjectRef):
             return str(out)
 
 
-def create_target_library_info(triple):
-    return TargetLibraryInfo(
-        ffi.lib.LLVMPY_CreateTargetLibraryInfo(_encode_string(triple, ))
-    )
-
-
-class TargetLibraryInfo(ffi.ObjectRef):
-    """
-    A LLVM TargetLibraryInfo.  Use :func:`create_target_library_info`
-    to create instances.
-    """
-
-    def _dispose(self):
-        self._capi.LLVMPY_DisposeTargetLibraryInfo(self)
-
-    def add_pass(self, pm):
-        """
-        Add this library info as a pass to PassManager *pm*.
-        """
-        ffi.lib.LLVMPY_AddTargetLibraryInfo(self, pm)
-        # Once added to a PassManager, we can never get it back.
-        self._owned = True
-
-    def disable_all(self):
-        """
-        Disable all "builtin" functions.
-        """
-        ffi.lib.LLVMPY_DisableAllBuiltins(self)
-
-    def get_libfunc(self, name):
-        """
-        Get the library function *name*.  NameError is raised if not found.
-        """
-        lf = c_int()
-        if not ffi.lib.LLVMPY_GetLibFunc(self, _encode_string(name),
-                                         byref(lf)):
-            raise NameError("LibFunc '{name}' not found".format(name=name))
-        return LibFunc(name=name, identity=lf.value)
-
-    def set_unavailable(self, libfunc):
-        """
-        Mark the given library function (*libfunc*) as unavailable.
-        """
-        ffi.lib.LLVMPY_SetUnavailableLibFunc(self, libfunc.identity)
-
-
-LibFunc = collections.namedtuple("LibFunc", ["identity", "name"])
-
 # ============================================================================
 # FFI
 
+ffi.lib.LLVMPY_GetProcessTriple.argtypes = [POINTER(c_char_p)]
+
+ffi.lib.LLVMPY_GetHostCPUFeatures.argtypes = [POINTER(c_char_p)]
+
 ffi.lib.LLVMPY_GetDefaultTargetTriple.argtypes = [POINTER(c_char_p)]
 
 ffi.lib.LLVMPY_GetHostCPUName.argtypes = [POINTER(c_char_p)]
@@ -368,34 +389,6 @@ ffi.lib.LLVMPY_GetBufferSize.restype = c_size_t
 
 ffi.lib.LLVMPY_DisposeMemoryBuffer.argtypes = [ffi.LLVMMemoryBufferRef]
 
-ffi.lib.LLVMPY_CreateTargetLibraryInfo.argtypes = [c_char_p]
-ffi.lib.LLVMPY_CreateTargetLibraryInfo.restype = ffi.LLVMTargetLibraryInfoRef
-
-ffi.lib.LLVMPY_DisposeTargetLibraryInfo.argtypes = [
-    ffi.LLVMTargetLibraryInfoRef,
-]
-
-ffi.lib.LLVMPY_AddTargetLibraryInfo.argtypes = [
-    ffi.LLVMTargetLibraryInfoRef,
-    ffi.LLVMPassManagerRef,
-]
-
-ffi.lib.LLVMPY_DisableAllBuiltins.argtypes = [
-    ffi.LLVMTargetLibraryInfoRef,
-]
-
-ffi.lib.LLVMPY_GetLibFunc.argtypes = [
-    ffi.LLVMTargetLibraryInfoRef,
-    c_char_p,
-    POINTER(c_int),
-]
-ffi.lib.LLVMPY_GetLibFunc.restype = c_int
-
-ffi.lib.LLVMPY_SetUnavailableLibFunc.argtypes = [
-    ffi.LLVMTargetLibraryInfoRef,
-    c_int,
-]
-
 ffi.lib.LLVMPY_GetTargetMachineData.argtypes = [
     ffi.LLVMTargetMachineRef,
 ]
