@@ -9,6 +9,7 @@ import sys
 import unittest
 import platform
 import locale
+import re
 
 from llvmlite import six
 from llvmlite import binding as llvm
@@ -166,6 +167,36 @@ class TestMisc(BaseTest):
         self.assertIsInstance(triple, str)
         self.assertTrue(triple)
 
+    def test_get_process_triple(self):
+        triple = llvm.get_process_triple()
+        default = llvm.get_default_triple()
+        self.assertIsInstance(triple, str)
+        self.assertTrue(triple)
+
+        default_parts = default.split('-')
+        triple_parts = triple.split('-')
+        # Arch must be equal
+        self.assertEqual(default_parts[0], triple_parts[0])
+
+    def test_get_host_cpu_features(self):
+        features = llvm.get_host_cpu_features()
+        self.assertIsInstance(features, dict)
+        self.assertIsInstance(features, llvm.FeatureMap)
+        for k, v in features.items():
+            self.assertIsInstance(k, str)
+            self.assertTrue(k)  # feature string cannot be empty
+            self.assertIsInstance(v, bool)
+        self.assertIsInstance(features.flatten(), str)
+
+        re_term = r"[+\-][a-zA-Z0-9\._]+"
+        regex = r"^({0}|{0}(,{0})*)?$".format(re_term)
+        # quick check for our regex
+        self.assertIsNotNone(re.match(regex, ""))
+        self.assertIsNotNone(re.match(regex, "+aa"))
+        self.assertIsNotNone(re.match(regex, "+a,-bb"))
+        # check CpuFeature.flatten()
+        self.assertIsNotNone(re.match(regex, features.flatten()))
+
     def test_get_host_cpu_name(self):
         cpu = llvm.get_host_cpu_name()
         self.assertIsInstance(cpu, str)
@@ -194,7 +225,7 @@ class TestMisc(BaseTest):
 
     def test_version(self):
         major, minor, patch = llvm.llvm_version_info
-        self.assertIn((major, minor), [(3, 5), (3, 6)])
+        self.assertIn((major, minor), [(3, 6), (3, 7)])
         self.assertIn(patch, range(10))
 
     def test_check_jit_execution(self):
@@ -741,42 +772,6 @@ class TestTargetMachine(BaseTest):
         self.assertEqual(td.get_abi_size(gv_i32.type), pointer_size)
 
 
-class TestTargetLibraryInfo(BaseTest):
-
-    def tli(self):
-        return llvm.create_target_library_info(llvm.get_default_triple())
-
-    def test_create_target_library_info(self):
-        tli = llvm.create_target_library_info(llvm.get_default_triple())
-        with tli:
-            pass
-        tli.close()
-
-    def test_get_libfunc(self):
-        tli = self.tli()
-        with self.assertRaises(NameError):
-            tli.get_libfunc("xyzzy")
-        fmin = tli.get_libfunc("fmin")
-        self.assertEqual(fmin.name, "fmin")
-        self.assertIsInstance(fmin.identity, int)
-        fmax = tli.get_libfunc("fmax")
-        self.assertNotEqual(fmax.identity, fmin.identity)
-
-    def test_set_unavailable(self):
-        tli = self.tli()
-        fmin = tli.get_libfunc("fmin")
-        tli.set_unavailable(fmin)
-
-    def test_disable_all(self):
-        tli = self.tli()
-        tli.disable_all()
-
-    def test_add_pass(self):
-        tli = self.tli()
-        pm = llvm.create_module_pass_manager()
-        tli.add_pass(pm)
-
-
 class TestPassManagerBuilder(BaseTest):
 
     def pmb(self):
